@@ -247,6 +247,7 @@ import_one( const char *fname, KBNODE keyblock )
     u32 keyid[2];
     int rc = 0;
     int new_key = 0;
+    int mod_key = 0;
 
     /* get the key and print some info about it */
     node = find_kbnode( keyblock, PKT_PUBLIC_KEY );
@@ -284,6 +285,7 @@ import_one( const char *fname, KBNODE keyblock )
 	return 0;
     }
 
+
     /* do we have this key already in one of our pubrings ? */
     pk_orig = m_alloc_clear( sizeof *pk_orig );
     rc = get_pubkey( pk_orig, keyid );
@@ -350,6 +352,7 @@ import_one( const char *fname, KBNODE keyblock )
 	if( rc )
 	    goto leave;
 	if( n_uids || n_sigs || n_subk ) {
+	    mod_key = 1;
 	    /* keyblock_orig has been updated; write */
 	    if( opt.verbose > 1 )
 		log_info_f(keyblock_resource_name(&kbpos),
@@ -357,7 +360,7 @@ import_one( const char *fname, KBNODE keyblock )
 	    if( (rc=lock_keyblock( &kbpos )) )
 		log_error_f(keyblock_resource_name(&kbpos),
 			 _("can't lock public keyring: %s\n"), g10_errstr(rc) );
-	    else if( (rc=update_keyblock( &kbpos, keyblock )) )
+	    else if( (rc=update_keyblock( &kbpos, keyblock_orig )) )
 		log_error_f( keyblock_resource_name(&kbpos),
 			    _("can't write keyblock: %s\n"), g10_errstr(rc) );
 	    unlock_keyblock( &kbpos );
@@ -394,6 +397,8 @@ import_one( const char *fname, KBNODE keyblock )
 		log_error("key %08lX: trustdb insert failed: %s\n",
 					(ulong)keyid[1], g10_errstr(rc) );
 	}
+	else if( mod_key )
+	    rc = update_trust_record( keyblock_orig );
 	else
 	    rc = clear_trust_checked_flag( new_key? pk : pk_orig );
     }
@@ -413,7 +418,6 @@ import_secret_one( const char *fname, KBNODE keyblock )
 {
     PKT_secret_key *sk;
     KBNODE node, uidnode;
-    KBNODE keyblock_orig = NULL;
     KBPOS kbpos;
     u32 keyid[2];
     int rc = 0;
@@ -459,7 +463,7 @@ import_secret_one( const char *fname, KBNODE keyblock )
 		      _("can't lock secret keyring: %s\n"), g10_errstr(rc) );
 	else if( (rc=insert_keyblock( &kbpos, keyblock )) )
 	    log_error_f(keyblock_resource_name(&kbpos),
-		      _("can't write keyring\n"), g10_errstr(rc) );
+		      _("can't write keyring: %s\n"), g10_errstr(rc) );
 	unlock_keyblock( &kbpos );
 	/* we are ready */
 	log_info_f(fname, _("key %08lX: secret key imported\n"), (ulong)keyid[1]);
@@ -472,7 +476,6 @@ import_secret_one( const char *fname, KBNODE keyblock )
 	log_error_f(fname, _("key %08lX: secret key not found: %s\n"),
 				(ulong)keyid[1], g10_errstr(rc));
 
-    release_kbnode( keyblock_orig );
     return rc;
 }
 
@@ -655,16 +658,17 @@ delete_inv_parts( const char *fname, KBNODE keyblock, u32 *keyid )
 	else if( node->pkt->pkttype == PKT_SIGNATURE
 		 && node->pkt->pkt.signature->sig_class == 0x20 )  {
 	    if( uid_seen ) {
-		log_error_f(fname, _("key %08lX: revocation certificate at wrong "
-			   "place - skipped\n"), fname, (ulong)keyid[1]);
+		log_error_f(fname, _("key %08lX: revocation certificate "
+				     "at wrong place - skipped\n"),
+				    (ulong)keyid[1]);
 		delete_kbnode( node );
 	    }
 	    else {
 		int rc = check_key_signature( keyblock, node, NULL);
 		if( rc ) {
-		    log_error_f(fname, _("key %08lX: invalid revocation certificate"
-			      ": %s - skipped\n"),
-			      fname, (ulong)keyid[1], g10_errstr(rc));
+		    log_error_f(fname, _("key %08lX: invalid revocation "
+			      "certificate: %s - skipped\n"),
+			      (ulong)keyid[1], g10_errstr(rc));
 		    delete_kbnode( node );
 		}
 	    }
@@ -788,7 +792,7 @@ append_uid( KBNODE keyblock, KBNODE node, int *n_sigs,
     /* at lease a self signature comes next to the user-id */
     if( node->next->pkt->pkttype == PKT_USER_ID ) {
 	log_error_f(fname, _("key %08lX: our copy has no self-signature\n"),
-						  fname, (ulong)keyid[1]);
+						  (ulong)keyid[1]);
 	return G10ERR_GENERAL;
     }
 
