@@ -99,8 +99,8 @@ int  build_sk_list( STRLIST locusr, SK_LIST *ret_sk_list,
 					    int unlock, unsigned usage );
 
 /*-- passphrase.h --*/
-void set_passphrase_fd( int fd );
-int  get_passphrase_fd(void);
+int  have_static_passphrase(void);
+void read_passphrase_from_fd( int fd );
 DEK *passphrase_to_dek( u32 *keyid, int cipher_algo, STRING2KEY *s2k, int mode);
 void set_next_passphrase( const char *s );
 char *get_last_passphrase(void);
