@@ -384,7 +384,8 @@ ask_algo( int *ret_v4, int addmode )
     if( !addmode )
 	tty_printf(_("   (%d) DSA and ElGamal (default)\n"), 1 );
     tty_printf(    _("   (%d) ElGamal (sign and encrypt)\n"), 2 );
-    tty_printf(    _("   (%d) ElGamal (encrypt only)\n"), 3 );
+    if( addmode )
+	tty_printf(    _("   (%d) ElGamal (encrypt only)\n"), 3 );
     tty_printf(    _("   (%d) DSA (sign only)\n"), 4 );
     tty_printf(    _("   (%d) ElGamal in a v3 packet\n"), 5 );
 
@@ -402,7 +403,7 @@ ask_algo( int *ret_v4, int addmode )
 	    algo = PUBKEY_ALGO_ELGAMAL;
 	    break;
 	}
-	else if( algo == 3 ) {
+	else if( algo == 3 && addmode ) {
 	    algo = PUBKEY_ALGO_ELGAMAL_E;
 	    break;
 	}
@@ -687,9 +688,9 @@ ask_passphrase( STRING2KEY **ret_s2k )
 
     s2k = m_alloc_secure( sizeof *s2k );
     for(;;) {
-	s2k->mode = 1;
-	s2k->hash_algo = DIGEST_ALGO_RMD160;
-	dek = passphrase_to_dek( NULL, CIPHER_ALGO_BLOWFISH, s2k, 2 );
+	s2k->mode = opt.s2k_mode;
+	s2k->hash_algo = opt.s2k_digest_algo;
+	dek = passphrase_to_dek( NULL, opt.s2k_cipher_algo, s2k, 2 );
 	if( !dek ) {
 	    tty_printf(_("passphrase not correctly repeated; try again.\n"));
 	}
@@ -951,7 +952,7 @@ generate_subkeypair( KBNODE pub_keyblock, KBNODE sec_keyblock )
 	break;
       default:
 	tty_printf("Key is protected.\n");
-	rc = check_secret_key( sk );
+	rc = check_secret_key( sk, 0 );
 	if( !rc )
 	    passphrase = get_last_passphrase();
 	break;
@@ -970,10 +971,10 @@ generate_subkeypair( KBNODE pub_keyblock, KBNODE sec_keyblock )
 
     if( passphrase ) {
 	s2k = m_alloc_secure( sizeof *s2k );
-	s2k->mode = 1;
-	s2k->hash_algo = DIGEST_ALGO_RMD160;
+	s2k->mode = opt.s2k_mode;
+	s2k->hash_algo = opt.s2k_digest_algo;
 	set_next_passphrase( passphrase );
-	dek = passphrase_to_dek( NULL, CIPHER_ALGO_BLOWFISH, s2k, 2 );
+	dek = passphrase_to_dek( NULL, opt.s2k_cipher_algo, s2k, 2 );
     }
 
     rc = do_create( algo, nbits, pub_keyblock, sec_keyblock,
