@@ -88,8 +88,10 @@ add_onepass_sig( CTX c, PACKET *pkt )
 	if( c->list->pkt->pkttype != PKT_ONEPASS_SIG ) {
 	   log_error("add_onepass_sig: another packet is in the way\n");
 	   release_list( c );
+	   c->list = new_kbnode( pkt );
 	}
-	add_kbnode( c->list, new_kbnode( pkt ));
+	else
+	   add_kbnode( c->list, new_kbnode( pkt ));
     }
     else /* insert the first one */
 	c->list = node = new_kbnode( pkt );
@@ -241,7 +243,9 @@ proc_plaintext( CTX c, PACKET *pkt )
     int any, clearsig, rc;
     KBNODE n;
 
-    if( opt.verbose )
+    if( pt->namelen == 8 && !memcmp( pt->name, "_CONSOLE", 8 ) )
+	log_info(_("note: sender requested \"for-your-eyes-only\"\n"));
+    else if( opt.verbose )
 	log_info("original file name='%.*s'\n", pt->namelen, pt->name);
     free_md_filter_context( &c->mfx );
     c->mfx.md = md_open( 0, 0);
@@ -844,7 +848,9 @@ check_sig_and_print( CTX c, KBNODE node )
 
     rc = do_check_sig(c, node, NULL );
     if( !rc || rc == G10ERR_BAD_SIGN ) {
-	write_status( rc? STATUS_BADSIG : STATUS_GOODSIG );
+	char *us = get_user_id_string( sig->keyid );
+	write_status_text( rc? STATUS_BADSIG : STATUS_GOODSIG, us );
+	m_free(us);
 	log_info(rc? _("BAD signature from \"")
 		   : _("Good signature from \""));
 	print_keyid( stderr, sig->keyid );
