@@ -811,6 +811,11 @@ parse_signature( IOBUF inp, int pkttype, unsigned long pktlen,
 
     if( is_v4 ) { /*extract required information */
 	const byte *p;
+
+	/* FIXME: set sig->flags.unknown_critical is there is a
+	 * critical bit set for packets which are not understood
+	 * It does only make sense for hashed data.
+	 */
 	p = parse_sig_subpkt( sig->hashed_data, SIGSUBPKT_SIG_CREATED, NULL );
 	if( !p )
 	    log_error("signature packet without timestamp\n");
@@ -1068,6 +1073,9 @@ parse_key( IOBUF inp, int pkttype, unsigned long pktlen,
 		    }
 		    sk->protect.s2k.count = iobuf_get(inp);
 		    pktlen--;
+		    if( list_mode )
+			printf("\tprotect count: %lu\n",
+					    (ulong)sk->protect.s2k.count);
 		}
 	    }
 	    else { /* old version; no S2K, so we set mode to 0, hash MD5 */
